@@ -25,14 +25,18 @@
 		     run-with-idle-timer mouse-minibuffer-check window-edges
 		     event-click-count track-mouse read-event mouse-movement-p
 		     event-end mouse-scroll-subr overlay-lists delete-overlay
-		     set-face-stipple mail-abbrevs-setup char-int
+		     set-face-stipple mail-abbrevs-setup 
 		     make-char-table set-char-table-range font-create-object
 		     x-color-values widget-make-intangible error-message-string
 		     w3-form-encode-xwfu gnus-mule-get-coding-system
 		     decode-coding-string mail-aliases-setup
 		     mm-copy-tree url-view-url w3-prepare-buffer
 		     set-buffer-multibyte
-		     find-non-ascii-charset-region char-charset))
+		     find-non-ascii-charset-region char-charset
+		     mule-write-region-no-coding-system
+		     find-charset-region base64-decode-string
+		     find-coding-systems-region get-charset-property
+		     coding-system-get))
       (maybe-bind '(global-face-data
 		    mark-active transient-mark-mode mouse-selection-click-count
 		    mouse-selection-click-count-buffer buffer-display-table
@@ -40,7 +44,7 @@
 		    gnus-newsgroup-name gnus-article-x-face-too-ugly
 		    mail-mode-hook enable-multibyte-characters
 		    adaptive-fill-first-line-regexp adaptive-fill-regexp
-		    url-current-mime-headers)))
+		    url-current-mime-headers buffer-file-coding-system)))
   (maybe-bind '(mail-mode-hook
 		enable-multibyte-characters browse-url-browser-function
 		adaptive-fill-first-line-regexp adaptive-fill-regexp
@@ -60,7 +64,8 @@
 		 gnus-mule-get-coding-system decode-coding-string
 		 mail-aliases-setup
 		 mm-copy-tree url-view-url w3-prepare-buffer
-		 char-int mule-write-region-no-coding-system)))
+		 mule-write-region-no-coding-system char-int
+		 base64-decode)))
 
 (setq load-path (cons "." load-path))
 (require 'custom)
