@@ -24,20 +24,8 @@
 
 ;;; Code:
 
-(eval-and-compile
-  (if (fboundp 'decode-coding-string)
-      (fset 'mm-decode-coding-string 'decode-coding-string)
-    (fset 'mm-decode-coding-string (lambda (s a) s))))
-
-(eval-and-compile
-  (if (fboundp 'encode-coding-string)
-      (fset 'mm-encode-coding-string 'encode-coding-string)
-    (fset 'mm-encode-coding-string (lambda (s a) s))))
-
-(eval-and-compile
-  (if (fboundp 'coding-system-list)
-      (fset 'mm-coding-system-list 'coding-system-list)
-    (fset 'mm-coding-system-list 'ignore)))
+(defvar mm-known-charsets '(iso-8859-1)
+  "List of known charsets.")
 
 (defvar mm-mime-mule-charset-alist
   '((us-ascii ascii)
@@ -76,6 +64,45 @@
 		    chinese-cns11643-7))
   "Alist of MIME-charset/MULE-charsets.")
 
+
+(eval-and-compile
+  (if (fboundp 'decode-coding-string)
+      (fset 'mm-decode-coding-string 'decode-coding-string)
+    (fset 'mm-decode-coding-string (lambda (s a) s)))
+
+  (if (fboundp 'encode-coding-string)
+      (fset 'mm-encode-coding-string 'encode-coding-string)
+    (fset 'mm-encode-coding-string (lambda (s a) s)))
+
+  (if (fboundp 'encode-coding-region)
+      (fset 'mm-encode-coding-region 'encode-coding-region)
+    (fset 'mm-encode-coding-region 'ignore))
+
+  (if (fboundp 'decode-coding-region)
+      (fset 'mm-decode-coding-region 'decode-coding-region)
+    (fset 'mm-decode-coding-region 'ignore))
+
+  (if (fboundp 'coding-system-list)
+      (fset 'mm-coding-system-list 'coding-system-list)
+    (fset 'mm-coding-system-list 'ignore))
+
+  (if (fboundp 'char-int)
+      (fset 'mm-char-int 'char-int)
+    (fset 'mm-char-int 'identity))
+
+  (if (fboundp 'coding-system-equal)
+      (fset 'mm-coding-system-equal 'coding-system-equal)
+    (fset 'mm-coding-system-equal 'equal))
+
+  (if (fboundp 'read-coding-system)
+      (fset 'mm-read-coding-system 'read-coding-system)
+    (defun mm-read-coding-system (prompt)
+      "Prompt the user for a coding system."
+      (completing-read
+       prompt (mapcar (lambda (s) (list (symbol-name (car s))))
+		      mm-mime-mule-charset-alist)))))
+
+
 (defvar mm-charset-coding-system-alist
   (let ((rest
 	 '((us-ascii . iso-8859-1)
@@ -92,6 +119,7 @@
     dest)
   "Charset/coding system alist.")
 
+
 (defun mm-mule-charset-to-mime-charset (charset)
   "Return the MIME charset corresponding to MULE CHARSET."
   (let ((alist mm-mime-mule-charset-alist)
@@ -118,7 +146,7 @@ used as the line break code type of the coding system."
   (cond
    ;; Running in a non-MULE environment.
    ((and (null (mm-coding-system-list))
-	 (eq charset 'iso-8859-1))
+	 (memq charset mm-known-charsets))
     charset)
    ;; Check to see whether we can handle this charset.
    ((memq charset (mm-coding-system-list))
@@ -139,6 +167,35 @@ used as the line break code type of the coding system."
       (setq idx (1+ idx)))
     string))
 
+(defun mm-enable-multibyte ()
+  "Enable multibyte in the current buffer."
+  (when (fboundp 'set-buffer-multibyte)
+    (set-buffer-multibyte t)))
+
+(defun mm-insert-rfc822-headers (charset encoding)
+  "Insert text/plain headers with CHARSET and ENCODING."
+  (insert "MIME-Version: 1.0\n")
+  (insert "Content-Type: text/plain; charset=\""
+	  (downcase (symbol-name charset)) "\"\n")
+  (insert "Content-Transfer-Encoding: "
+	  (downcase (symbol-name encoding)) "\n"))
+
+(defun mm-content-type-charset (header)
+  "Return the charset parameter from HEADER."
+  (when (string-match "charset *= *\"? *\\([-0-9a-zA-Z_]+\\)\"? *$" header)
+    (intern (downcase (match-string 1 header)))))
+
+
+(defun mm-mime-charset (charset b e)
+  (if (fboundp 'coding-system-get)
+      (or
+       (coding-system-get
+	(get-charset-property charset 'prefered-coding-system)
+	'mime-charset)
+       (car (memq charset (find-coding-systems-region
+			   (point-min) (point-max)))))
+    (mm-mule-charset-to-mime-charset charset)))
+
 (provide 'mm-util)
 
 ;;; mm-util.el ends here
